@@ -1,19 +1,13 @@
-# encoding: utf-8
-
 """
 .. codeauthor:: Tsuyoshi Hombashi <tsuyoshi.hombashi@gmail.com>
 """
 
-from __future__ import print_function, unicode_literals
-
 import itertools
 import sys
 from collections import OrderedDict, namedtuple
 from decimal import Decimal
 
 import pytest
-import six
-from six.moves import zip
 from typepy import Integer, String
 
 from tabledata import DataError, PatternMatch, TableData
@@ -37,7 +31,7 @@ def dumps_results(expected=None, actual=None):
         print("actual: {}".format(dumps_tabledata(actual)))
 
 
-class Test_TableData_constructor(object):
+class Test_TableData_constructor:
     @pytest.mark.parametrize(
         ["table_name", "headers", "rows", "expected"],
         [
@@ -103,11 +97,10 @@ class Test_TableData_constructor(object):
 def yield_rows():
     rows = [[1, 2], [3, 4]]
 
-    for row in rows:
-        yield row
+    yield from rows
 
 
-class Test_TableData_num_rows(object):
+class Test_TableData_num_rows:
     @pytest.mark.parametrize(
         ["table_name", "headers", "rows", "expected"],
         [
@@ -125,7 +118,7 @@ class Test_TableData_num_rows(object):
         assert table_data.num_rows == expected
 
 
-class Test_TableData_eq(object):
+class Test_TableData_eq:
 
     __DATA_0 = TableData(
         "Sheet1",
@@ -170,23 +163,31 @@ class Test_TableData_eq(object):
         assert (lhs != rhs) == (not expected)
 
 
-class Test_TableData_equals(object):
+class Test_TableData_equals:
 
     __LHS = TableData("tablename", ["a", "b"], [{"a": 1, "b": 2}, {"a": 11, "b": 12}])
     __RHS = TableData("tablename", ["a", "b"], [[1, 2], [11, 12]])
 
     @pytest.mark.parametrize(
         ["lhs", "rhs", "cmp_by_dp", "expected"],
-        [[__LHS, __RHS, False, True], [__LHS, __RHS, True, False]],
+        [[__LHS, __RHS, True, True], [__LHS, __RHS, False, False]],
     )
     def test_normal(self, lhs, rhs, cmp_by_dp, expected):
+        empty_td = TableData("tablename", ["a", "b"], None)
+
         assert lhs.equals(rhs, cmp_by_dp=cmp_by_dp) == expected
-        assert lhs.in_tabledata_list([rhs], cmp_by_dp=cmp_by_dp) == expected
-        assert lhs.in_tabledata_list([lhs], cmp_by_dp=cmp_by_dp)
-        assert lhs.in_tabledata_list([rhs, lhs], cmp_by_dp=cmp_by_dp)
+        assert lhs.equals(empty_td, cmp_by_dp=cmp_by_dp) is False
+        assert empty_td.equals(rhs, cmp_by_dp=cmp_by_dp) is False
+        assert (lhs == rhs) is False
+        assert (lhs != rhs) is True
+
+        assert lhs.in_tabledata_list([rhs, empty_td], cmp_by_dp=cmp_by_dp) == expected
+        assert lhs.in_tabledata_list([lhs, empty_td], cmp_by_dp=cmp_by_dp)
+        assert lhs.in_tabledata_list([rhs, lhs, empty_td], cmp_by_dp=cmp_by_dp)
+        assert empty_td.in_tabledata_list([rhs, lhs], cmp_by_dp=cmp_by_dp) is False
 
 
-class Test_TableData_repr(object):
+class Test_TableData_repr:
     @pytest.mark.parametrize(
         ["table_name", "headers", "rows", "expected"],
         [
@@ -215,10 +216,10 @@ class Test_TableData_repr(object):
     def test_normal(self, table_name, headers, rows, expected):
         tabledata = TableData(table_name, headers, rows)
 
-        assert six.text_type(tabledata) == expected
+        assert str(tabledata) == expected
 
 
-class Test_TableData_as_dict(object):
+class Test_TableData_as_dict:
     @pytest.mark.parametrize(
         ["table_name", "headers", "rows", "expected"],
         [
@@ -258,7 +259,7 @@ class Test_TableData_as_dict(object):
         assert TableData(table_name, headers, rows).as_dict() == expected
 
 
-class Test_TableData_as_tuple(object):
+class Test_TableData_as_tuple:
     @pytest.mark.parametrize(
         ["table_name", "headers", "rows", "expected"],
         [
@@ -287,7 +288,7 @@ class Test_TableData_as_tuple(object):
             assert tuple(lhs) == rhs
 
 
-class Test_TableData_transpose(object):
+class Test_TableData_transpose:
     @pytest.mark.parametrize(
         ["value", "expected"],
         [
@@ -301,7 +302,7 @@ class Test_TableData_transpose(object):
         assert value.transpose() == expected
 
 
-class Test_TableData_value_dp_matrix(object):
+class Test_TableData_value_dp_matrix:
 
     __MIXED_DATA = [
         [1, 2],
@@ -354,7 +355,7 @@ class Test_TableData_value_dp_matrix(object):
         assert tabledata.has_value_dp_matrix
 
 
-class Test_TableData_is_empty_header(object):
+class Test_TableData_is_empty_header:
     @pytest.mark.parametrize(
         ["table_name", "headers", "rows", "expected"],
         [["tablename", [], [], True], ["tablename", ["a", "b"], [], False]],
@@ -365,7 +366,7 @@ class Test_TableData_is_empty_header(object):
         assert tabledata.is_empty_header() == expected
 
 
-class Test_TableData_is_empty_rows(object):
+class Test_TableData_is_empty_rows:
     @pytest.mark.parametrize(
         ["table_name", "headers", "rows", "expected"],
         [
@@ -380,7 +381,7 @@ class Test_TableData_is_empty_rows(object):
         assert tabledata.is_empty_rows() == expected
 
 
-class Test_TableData_is_empty(object):
+class Test_TableData_is_empty:
     @pytest.mark.parametrize(
         ["table_name", "headers", "rows", "expected"],
         [
@@ -395,7 +396,7 @@ class Test_TableData_is_empty(object):
         assert tabledata.is_empty() == expected
 
 
-class Test_TableData_validate_rows(object):
+class Test_TableData_validate_rows:
     @pytest.mark.parametrize(
         ["table_name", "headers", "rows"],
         [["tablename", [], []], ["tablename", ["a", "b"], []], ["tablename", ["a", "b"], [[1, 2]]]],
@@ -415,7 +416,7 @@ class Test_TableData_validate_rows(object):
             TableData(table_name, headers, rows).validate_rows()
 
 
-class Test_TableData_filter_column(object):
+class Test_TableData_filter_column:
     HEADERS = ["abcde", "test"]
     VALUE_MATRIX = [[1, 2], [3, 4]]
 
